# -*- coding: utf-8 -*-

"""
    iiif2
    ~~~~~
    An Image Processing Library for the IIIF Image API 2.0
    Specification.

    :copyright: (c) 2015 by Authors
    :license: see LICENSE for more details.
"""

__title__ = 'iiif2'
__version__ = '0.0.32'
__author__ = [
    'Mek <michael.karpeles@gmail.com>'
    ]

from .iiif import IIIF  # NOQA
from .web import Parse  # NOQA
